	include	tios.h
	include	flib.h
	include	hexlib.h

	include genlib.h
	include macros.h

	xdef	_main
	xdef	_comment

	ifne	CALC_TI92PLUS
	xdef	_ti92plus
	xdef	_ti89
	endif

_main:
	jsr	genlib::init		; Init the grey-mode (or gray-mode ?)
	genlib::PUSH_DSCREEN	d0	; Allocate a DSCREEN on the stack (7688 bytes)
	move.l	d0,Screen		; Save its adress
					; you can't allocate more than one screen

	jsr	genlib::init_dscreen	; Allocat a new DScreen for double buffer
	tst.w	d0
	beq	memory_error		; Memory ?
					; Ok save handle
	jsr	genlib::push_hd		; in the internal handle list (max = 20 handles!)
					; No need to test d1
	move.l	a0,Screen2		; and the adress

	move.l	Screen(PC),d0
	jsr	genlib::set_dscreen_int		; Set the interrupt to this Dscreen
	move.l	Screen2(PC),d0
	jsr	genlib::set_dscreen_function	; Set the functions to this DScreen

	jsr	genlib::change_update	; Use the real size

	lea	Plane_Table(PC),a3	; Plane Table
	sub.l	a5,a5			; No indirect table
	lea	Plane_Tile(Pc),a4	; Tiles
	moveq	#32,d3			; Size of a row
	jsr	genlib::init_plane	; Init the plane
	tst.w	d0			; test : memory ?
	beq	memory_error		; Yes
	jsr	genlib::push_hd		; Save handle
	move.l	a0,plane		; Save the adress of the plane
	
	lea	Plane_Table2(PC),a3	; Plane 2 table
	sub.l	a5,a5			; No indirect table
	lea	Plane_Tile(Pc),a4	; Tiles adress
	moveq	#16,d3			; Size of a row
	jsr	genlib::init_plane	; init the plane
	tst.w	d0			; Test : memory ?
	beq	memory_error		; Yes
	jsr	genlib::push_hd		; Save handle
	move.l	a0,plane2		; Save the adress of the plane

	move.l	plane(pc),a0		; Select the plane
	move.w	#0,PLANE_XS(a0)		; Set the Scrolling coordinates
	move.w	#0,PLANE_YS(a0)

	move.l	plane2(pc),a0		; Select the plane
	move.w	#0,PLANE_XS(a0)		; Set the Scrolling coordinates
	move.w	#0,PLANE_YS(a0)

	move.w	#3,dir		; Direction x of plane 1
	move.w	#2,dir2		; Direction y of plane 1

	move.w	#1,dir3		; Direction x of plane 2
	move.w	#1,dir4		; Direction y of plane 2

	clr.w	temps		; Clear temps of double-buffer

\Continue:
; Change the coordinates of the scrolling of plane 1
	move.l	plane(pc),a0
	move.w	PLANE_XS(a0),d0
	add.w	dir(pc),d0
	cmp.w	#32*16-240,d0
	bls	\ok
	neg.w	dir
	bra.s	\pas_ok
\ok:
	move.w	d0,PLANE_XS(a0)
\pas_ok:
	move.w	PLANE_YS(a0),d0
	add.w	dir2(pc),d0
	cmp.w	#32*16-128,d0	
	bls	\ok2
	neg.w	dir2
	bra.s	\pas_ok2
\ok2:
	move.w	d0,PLANE_YS(a0)
\pas_ok2:

; Change the coordinates of the scrolling of plane 2
	move.l	plane2(pc),a0
	move.w	PLANE_XS(a0),d0
	add.w	dir3(pc),d0
	cmp.w	#16*16-240,d0
	bls	\ok3
	neg.w	dir3
	bra.s	\pas_ok3
\ok3:
	move.w	d0,PLANE_XS(a0)
\pas_ok3:
	move.w	PLANE_YS(a0),d0
	add.w	dir4(pc),d0
	cmp.w	#16*16-128,d0	
	bls	\ok4
	neg.w	dir4
	bra.s	\pas_ok4
\ok4:
	move.w	d0,PLANE_YS(a0)
\pas_ok4:

; Put the plane 2 as background
	move.l	plane2(pc),a0
	jsr	genlib::update_vscreen_16
	ifne	CALC_TI92PLUS
	tst.b	CALCULATOR
	beq.s	\89a
	jsr	genlib::put_plane	; TI-92 +
	bra.s	\donea
\89a:	jsr	genlib::put_plane_89	; TI-89 opt
\donea:
	endif
	ifeq	CALC_TI92PLUS
	jsr	genlib::put_plane	; TI-92
	endif

; Put the plane 1 as foreground
	move.l	plane(pc),a0
	jsr	genlib::update_vscreen_16
	ifne	CALC_TI92PLUS
	tst.b	CALCULATOR
	beq.s	\89b
	jsr	genlib::put_fgrd_plane
	bra.s	\doneb
\89b:	jsr	genlib::put_fgrd_plane_89
\doneb:
	endif
	ifeq	CALC_TI92PLUS
	jsr	genlib::put_fgrd_plane
	endif

; Do the double-buffer
	bsr	double_buffer

; Wait next 15Hz
\wait:	cmp.l	#5,genlib::timer
	bls.s	\wait			; 15 Hz waiting
	clr.l	genlib::timer

; A key ?
	jsr	genlib::keypressed	; A key ?
	tst.w	d0
	beq	\Continue		; no key

memory_error:
	jsr	genlib::free_hd		; Free all saved handle in reverse order.(FIFO)
	genlib::POP_DSCREEN		; Pop DScreen
	jsr	genlib::quit		; Disable grey-mode
	rts				; Quit

double_buffer:
	move.w	temps(pc),d1		; Read temps
	move.l	Screen(pc,d1.w),d0	; Read a screen
	jsr	genlib::set_dscreen_int	; Set the interrupt to this Dscreen
	eor.w	#4,d1			; Next screen
	move.l	Screen(PC,d1.w),d0	; Read the other screen
	jsr	genlib::set_dscreen_function	; Set the functions to this DScreen
	move.w	d1,temps		; Save temps
	rts

MyHd	dc.w	0		; Handle of double-buffer
Screen	dc.l	0		; Screen 1
Screen2	dc.l	0		; Screen 2
temps	dc.w	0		; Temps
plane	dc.l	0		; Plane 1
plane2	dc.l	0		; Plane 2
dir	dc.w	0		; Dir x1
dir2	dc.w	0		; Dir y1
dir3	dc.w	0		; Dir x2
dir4	dc.w	0		; Dir y2

Plane_Tile:
	incbin "decor.bin"

Plane_Table2:
	incbin "pl2.lvl"

Plane_Table:
	incbin "pl.lvl"

_comment
	dc.b	"Example 4",0
	end
