	include	tios.h
	include genlib.h

	xdef	_main
	xdef	_comment

	ifne	CALC_TI92PLUS		; Code a rajouter si on compile pour 92+/89
	xdef	_ti92plus
	xdef	_ti89
	endif				; Fin de compilation conditionnelle.

_main:
	; Initialisation de Genlib
	jsr	genlib::init		; Initialisation de Genlib

	; Allocation des ecrans DScreen
	genlib::PUSH_DSCREEN	d0	; Alloue a DScreen sur la pile
	move.l	d0,DScreen		; Sauvegarde son adresse

	jsr	genlib::init_dscreen	; Alloue un autre DScreen pour le double-buffer
	tst.w	d0
	beq	memory_error		; Memoire insuffisante pour l'allouer  ?
	jsr	genlib::push_hd		; Sauvegarde l'handle (=d0.w) dans la pile interne
					; d'handle de genlib.
	move.l	a0,DScreen2		; et l'adresse

	; Initialisation des DScreens
	move.l	DScreen(PC),d0
	jsr	genlib::set_dscreen_int		; On regardera cette ecran
	move.l	DScreen2(PC),d0
	jsr	genlib::set_dscreen_function	; On travaillera sur cette ecran.

	; Creation d'un plan
	lea	Plane_Table(PC),a3	; Plane Table
	suba.l	a5,a5			; pour compatibilite avec les futures versions de genlib
	lea	Plane_Tile(Pc),a4	; Adresse de la table des Tiles
	moveq	#5,d3			; Une ligne fait 2^5 = 32 colonnes
	jsr	genlib::init_plane	; Init le plan
	tst.w	d0			; test : Erreur de memoire ?
	beq	memory_error		; Oui, on quitte !
	jsr	genlib::push_hd		; Sinon on sauvegarde l'handle dans la pile interne
	move.l	a0,plane		; Et son adresse
	
	; Fixe les coordonnes du plan
	move.l	plane(pc),a0		; Adresse du plan
	move.w	#0,PLANE_XS(a0)		; Fixe ces coordonnes
	move.w	#0,PLANE_YS(a0)

	clr.w	temps			; Varaible pour double-buffer

\Continue:
	; Met a jour les coordonnes du plan
	move.l	plane(Pc),a0
	move.w	PLANE_YS(a0),d7
	move.w	PLANE_XS(a0),d6

	; Lecture clavier
	jsr	genlib::read_joypad

	btst.l	#exit_key,d0
	beq	memory_error		; Esc => On quitte
	; Gestion de Haut/Bas/Gauche/Droite
	btst.l	#right_key,d0
	bne.s	\no_right
	addq.w	#3,d6
\no_right:
	btst.l	#left_key,d0
	bne.s	\no_left
	subq.w	#3,d6
\no_left:
	btst.l	#up_key,d0
	bne.s	\no_up
	subq.w	#3,d7
\no_up:
	btst.l	#down_key,d0
	bne.s	\no_down
	addq.w	#3,d7
\no_down:
	; Sauvegarde les nouvelles coordonnes
	move.l	plane(pc),a0		; A recharger car detruit par read_joypad (cf doc)
	move.w	d6,PLANE_XS(a0)
	move.w	d7,PLANE_YS(a0)


	; On affiche le plan en tant qu'arriere plan
	; Pas besoin d'effacer l'ecran car il va etre entierement redessine
	move.l	plane(pc),a0		; On pourrait l'enlever car il a deja ete charge.
					; Neamoins, je le laisse pour des raisons pedagogiques
	jsr	genlib::update_vscreen_16	; Mise a jour du VScreen

	ifne	CALC_TI92PLUS		; Pour Doors
	tst.b	CALCULATOR		; optimisation pour la calc
	beq.s	\89a			; Rem : Affiche en ti-92+ sur 1 89 marche parfaitement
	jsr	genlib::put_plane	; TI-92 +
	bra.s	\donea
\89a:	jsr	genlib::put_plane_89	; TI-89
\donea:
	endif

	ifeq	CALC_TI92PLUS		; Pour Fargo
	jsr	genlib::put_plane	; TI-92
	endif

; Do the double-buffer
	move.w	temps(pc),d1		; Read temps
	move.l	DScreen(pc,d1.w),d0	; Read a screen
	jsr	genlib::set_dscreen_int	; Set the interrupt to this Dscreen
	eor.w	#4,d1			; Next screen
	move.l	DScreen(PC,d1.w),d0	; Read the other screen
	jsr	genlib::set_dscreen_function	; Set the functions to this DScreen
	move.w	d1,temps		; Save temps

	; On attend pour avoir une frequence de l'ordre de 30 Hz
\wait:	cmp.l	#2,genlib::timer
	bls.s	\wait
	clr.l	genlib::timer

	bra	\Continue		; On boucle

memory_error:
	jsr	genlib::free_hd		; Libere tous les handles de la pile interne de genlib
	genlib::POP_DSCREEN		; On desalloue le DScreen sur la pile
	jsr	genlib::quit		; On quitte genlib
	rts				; On quitte le programme

DScreen		dc.l	0		; DScreen 1
DScreen2	dc.l	0		; DScreen 2
temps	dc.w	0		; Temps
plane	dc.l	0		; Plane

Plane_Tile:
	incbin "decor.bin"		; Les decors
					; Editable grace a sprmaker (Attention au tile vide !)
Plane_Table:
	incbin "pl.lvl"			; Matrice de 32x32 representant la carte
					; Editable grace a plane maker
_comment
	dc.b	"Exemple de scrolling",0
	end

